#include "addnewtestwidget.h"
#include "ui_addnewtestwidget.h"
#include <QDebug>
#include <QSqlQueryModel>
#include <QSqlQuery>


addNewTestWidget::addNewTestWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::addNewTestWidget)
{
    ui->setupUi(this);
    QFont font = ui->lbl1->font();
    font.setPointSize(12);
    ui->lbl1->setFont(font);
    this->returnButton = new QPushButton("В главное меню", this);
    returnButton->show();
    this->setWindowTitle("Добавление нового теста");
    connect(this, SIGNAL(onTestEditStart()), this, SLOT(getTestName()));
}

addNewTestWidget::~addNewTestWidget()
{
    delete ui;
}

void addNewTestWidget::setDb(QSqlDatabase &p_db)
{
    db = p_db;
}

void addNewTestWidget::addNewTest()
{
    testEntity = new TestEntity();
    ui->textBrowser->hide();
    ui->label->hide();
    ui->label_2->hide();
    ui->label_3->hide();
    ui->variantsEdit->hide();
    ui->answerEdit->hide();
    ui->daleeButton->hide();

    ui->lbl1->setText("Введите имя теста");

    emit(onTestEditStart());
}

void addNewTestWidget::hidePrev()
{
    ui->lbl1->hide();
    ui->testNameInput->hide();
    ui->submitBtn->hide();
}

void addNewTestWidget::getTestName()
{
    connect(ui->submitBtn, SIGNAL(clicked()), this, SLOT(getQuestions()));
}

void addNewTestWidget::getQuestions()
{
    QString name = ui->testNameInput->text();
    this->testEntity->setName(name);
    qDebug() << testEntity->getName();

    ui->testNameInput->clear();

    ui->lbl1->setText("Введите вопрос, варианты \n и правильный ответ");

    ui->label->show();
    ui->label_2->show();
    ui->label_3->show();
    ui->variantsEdit->show();
    ui->answerEdit->show();
    ui->daleeButton->show();

    disconnect(ui->submitBtn, SIGNAL(clicked()), this, SLOT(getQuestions()));
    connect(ui->daleeButton, SIGNAL(clicked()), this, SLOT(showTest()));
    connect(ui->submitBtn, SIGNAL(clicked()), this, SLOT(getCurrentQuestion()));

    testEntity->setQuestions(questions);
}

void addNewTestWidget::getCurrentQuestion()
{
    counterQue++;

    QString question = ui->testNameInput->text();

    QString variantsString = ui->variantsEdit->text();
    QStringList variants = variantsString.split("%");

    QString answersString = ui->answerEdit->text();
    QStringList answersList = answersString.split(" ");
    QSet<QString> answers(answersList.begin(), answersList.end());

    QSqlQueryModel model(this);



    model.setQuery("SELECT MAX(id) FROM test_questions", db);

    int maximum = model.index(0,0).data().toInt() + 1;
    int id = maximum + counterQue;

    QuestionEntity quest(id, question, answers, variants);

    qDebug() << quest.getId() << " " << quest.getQuestion();

    questions.push_back(quest);

    counter++;

    ui->testNameInput->clear();
    ui->answerEdit->clear();
    ui->variantsEdit->clear();
}

void addNewTestWidget::showTest()
{
    testEntity->setQuestions(questions);
    disconnect(ui->submitBtn, SIGNAL(clicked()), this, SLOT(getCurrentQuestion()));
    ui->label->hide();
    ui->label_2->hide();
    ui->label_3->hide();
    ui->variantsEdit->hide();
    ui->answerEdit->hide();
    ui->testNameInput->hide();

    QFont font = ui->textBrowser->font();
    font.setPointSize(12);
    font.setStyleHint(QFont::OldEnglish);
    ui->textBrowser->setFont(font);
    ui->textBrowser->show();

    QSqlQueryModel model(this);
    model.setQuery("SELECT MAX(id) FROM tests", db);
    int id = model.index(0,0).data().toInt() + 1;
    testEntity->setId(id);

    QString testString = testEntity->getName() + "\n";
    for(QuestionEntity& qe: testEntity->getQuestions()){
        testString+= "\n" + qe.getQuestion() + "\n";
        int varCounter = 1;
        for(auto& el:qe.getVariants()){
            testString+=QString::number(varCounter) + ") " + el +"\n";
            varCounter++;
        }
         testString += "Правильный ответ: ";
        for(auto& el: qe.getAnswers()){
            testString += el + " ";
        }
        testString += "\n";
    }

    qDebug() << testString;

    ui->textBrowser->setText(testString);

    ui->lbl1->setText("Проверьте Ваш новый тест \n");

    ui->daleeButton->setText("Удалить тест и вернуться в главное меню");
    ui->submitBtn->setText("Отправить тест");

    disconnect(ui->daleeButton, SIGNAL(clicked()), this, SLOT(showTest()));
    disconnect(ui->submitBtn, SIGNAL(clicked()), this, SLOT(getCurrentQuestion()));

    connect(ui->daleeButton, SIGNAL(clicked()), this, SLOT(dontSaveTest()));
    connect(ui->submitBtn, SIGNAL(clicked()), this, SLOT(saveTest()));
}

void addNewTestWidget::dontSaveTest()
{
    delete(testEntity);
    this->hide();
}

void addNewTestWidget::saveTest()
{
    message = new QMessageBox();
    message->setText("Тест сохранен!");
    message->show();

    QSqlQuery query(db);

    query.exec("INSERT INTO tests VALUES(" + QString::number(testEntity->getId()) +
                                        ",'" + testEntity->getName() + "')");

    for(QuestionEntity& que: testEntity->getQuestions()){
        QString questionVariants;

        int variantsSize = que.getVariants().size();

        for(int i = 0; i < variantsSize - 1; i++){
            questionVariants += que.getVariants().at(i) + "%";
        }
        questionVariants += que.getVariants().at(variantsSize - 1);
        qDebug() << questionVariants;
        QString questionAnswers;

        for(auto& ans: que.getAnswers()){
            questionAnswers += ans + " ";
        }
        questionAnswers.chop(1);
        qDebug() << questionAnswers;

        query.exec("INSERT INTO test_questions VALUES (" + QString::number(que.getId()) +
                   ",'" + que.getQuestion() + "','" + questionVariants + "','" +
                   questionAnswers + "'," + QString::number(testEntity->getId()) + ");");

        qDebug() <<"INSERT INTO test_questions VALUES (" + QString::number(que.getId()) +
                   ",'" + que.getQuestion() + "','" + questionVariants + "','" +
                   questionAnswers + "'," + QString::number(testEntity->getId()) + ");";
    }

    this->hide();
    emit onClosed();
}

